namespace WICS.Generator.Processors
{
	using System;
	using System.Collections.Generic;
	using System.Text;

	using Microsoft.Build.Framework;

	/// <summary>
	/// Processes a file as a service.
	/// </summary>
	public class ServicesProcessor : Processor
	{
		/// <summary>
		/// Initializes the object.
		/// </summary>
		/// <param name="file">The file to process as an <see cref="ITaskItem"/>.</param>
		/// <param name="dirId">Sets the parent directories Id.</param>
		public ServicesProcessor ( ITaskItem file, string dirId ) : base ( file, dirId ) { }

		/// <summary>
		/// Processes the file when called.
		/// </summary>
		/// <param name="writer">An <see cref="XmlWriter"/> to append to.</param>
		public override void Execute ( System.Xml.XmlWriter writer )
		{
			this.WriteFile ( writer );
			this.WriteServiceInstall ( writer );
		}

		private void WriteServiceInstall ( System.Xml.XmlWriter writer )
		{
			writer.WriteStartElement ( "ServiceInstall" );
			this.WriteId ( writer );
			this.WriteAttributeFromMetadata ( writer, "Account" );
			this.WriteAttributeFromMetadata ( writer, "Arguments" );
			this.WriteAttributeFromMetadata ( writer, "Description" );
			this.WriteAttributeFromMetadata ( writer, "DisplayName" );
			this.WriteAttributeFromMetadata ( writer, "EraseDescription" );
			this.WriteAttributeFromMetadata ( writer, "ErrorControl" );
			this.WriteAttributeFromMetadata ( writer, "Interactive" );
			this.WriteAttributeFromMetadata ( writer, "LoadOrderGroup" );
			this.WriteAttributeFromMetadata ( writer, "Name" );
			this.WriteAttributeFromMetadata ( writer, "Password" );
			this.WriteAttributeFromMetadata ( writer, "Start" );
			this.WriteAttributeFromMetadata ( writer, "Type" );
			this.WriteAttributeFromMetadata ( writer, "Vital" );
			writer.WriteEndElement ( );
		}

		private void WriteId ( System.Xml.XmlWriter writer )
		{
			string id = this.GetMetadata ( "ServiceId" );
			if ( id == null )
				id = Common.Guids.CreateGuid ( "ServiceId", GuidManager.GuidType.Id );

			writer.WriteAttributeString ( "Id", id );
		}

		private void WriteAttributeFromMetadata ( System.Xml.XmlWriter writer, string attribute )
		{
			string str = this.GetMetadata ( "Service" + attribute );

			if ( str != null )
				writer.WriteAttributeString ( attribute, str );
		}

		private string GetMetadata ( string key )
		{
			string str = this.File.GetMetadata ( key );

			if ( !string.IsNullOrEmpty ( str ) )
				return str;

			return null;
		}
	}
}